@@ -1,10 +1,11 @@
+from abc import abstractmethod
 import asyncio
 import os
 from typing import Dict, List, Optional, Union
 from uuid import uuid4
 from .executor import Executor
 from .training import Training
-from .model import BasicModel
+from .model import BasicModel, PretrainedModel
 from ..context import Context
 from ..node import Node
 from .downloader import TrainingsDownloader
@@ -12,6 +13,7 @@ from ..rest import downloads, uploads
 from .. import node_helper
 import logging
 from icecream import ic
+from .helper import is_valid_uuid4
 
 
 class Trainer():
@@ -23,21 +25,34 @@ class Trainer():
 
     async def begin_training(self, context: Context, source_model: dict) -> None:
         downloader = TrainingsDownloader(context)
-
         self.training = Trainer.generate_training(context, source_model)
         self.training.data = await downloader.download_training_data(self.training.images_folder)
         self.executor = Executor(self.training.training_folder)
-        logging.info(f'downloading model {source_model["id"]} as {self.model_format}')
-        await downloads.download_model(self.training.training_folder, context, source_model['id'], self.model_format)
-        logging.info(f'now starting training')
-        await self.start_training()
+
+        id = source_model['id']
+        if not is_valid_uuid4(id):
+            if id in [m.name for m in self.provided_pretrained_models]:
+                logging.debug('Should start with pretrained model')
+                await self.start_training_from_scratch(id)
+            else:
+                raise ValueError(f'Pretrained model {id} is not supported')
+        else:
+            logging.debug('Should start with loop model')
+            logging.info(f'downloading model {source_model["id"]} as {self.model_format}')
+            await downloads.download_model(self.training.training_folder, context, source_model['id'], self.model_format)
+            logging.info(f'now starting training')
+            await self.start_training()
 
     async def start_training(self) -> None:
         raise NotImplementedError()
 
+    async def start_training_from_scratch(self, identifier: str) -> None:
+        raise NotImplementedError()
+
     def stop_training(self) -> bool:
         if self.executor:
             self.executor.stop()
+            self.executor = None
             return True
         else:
             logging.info('could not stop training, executor is None')
@@ -94,13 +109,17 @@ class Trainer():
         '''
         raise NotImplementedError()
 
+    @property
+    @abstractmethod
+    def provided_pretrained_models(self) -> List[PretrainedModel]:
+        raise NotImplementedError()
+
     @staticmethod
     def generate_training(context: Context, source_model: dict) -> Training:
         training_uuid = str(uuid4())
         project_folder = Node.create_project_folder(context)
         return Training(
             id=training_uuid,
-            base_model=source_model,
             context=context,
             project_folder=project_folder,
             images_folder=node_helper.create_image_folder(project_folder),
